@@ -67,6 +67,16 @@ typedef enum StandardAbsSyncDirection StandardAbsSyncDirection;
 #define LOG_CASE18 _("arhive and change conflict, pilot record saved locally\n")
 #define LOG_CASE20 _("merge conflict, pilot and local record swapped\n")
 
+#define Result(v) result = v; break
+
+#define Finish                                                             \
+if (result == 0) {                                                         \
+	standard_abs_close_db_and_purge_local (conduit, dbinfo, TRUE);     \
+} else {                                                                   \
+	standard_abs_close_db_and_purge_local (conduit, dbinfo, FALSE);    \
+}                                                                          \
+return result
+
 static void gnome_pilot_conduit_standard_abs_init		(GnomePilotConduitStandardAbs		 *pilot_conduit_standard_abs);
 static void gnome_pilot_conduit_standard_abs_class_init		(GnomePilotConduitStandardAbsClass	 *klass);
 static gint gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *conduit,
@@ -87,8 +97,8 @@ gint standard_abs_merge_to_local (GnomePilotConduitStandardAbs *, int, int, Stan
 int SlowSync (int, int, GnomePilotConduitStandardAbs *);
 gint FastSync (int, int, GnomePilotConduitStandardAbs *);
 gboolean gpilot_sync_pc_match(GnomePilotDBInfo*);
-void standard_abs_open_db(GnomePilotConduitStandardAbs*,GnomePilotDBInfo*);
-void standard_abs_close_db_and_purge_local(GnomePilotConduitStandardAbs*,GnomePilotDBInfo*);
+gint standard_abs_open_db(GnomePilotConduitStandardAbs*,GnomePilotDBInfo*);
+void standard_abs_close_db_and_purge_local(GnomePilotConduitStandardAbs*,GnomePilotDBInfo*, gboolean);
 
 static GnomePilotConduitStandardClass *parent_class = NULL;
 static guint pilot_conduit_standard_abs_signals[LAST_SIGNAL] = { 0 };
@@ -122,13 +132,14 @@ GtkObject *
 gnome_pilot_conduit_standard_abs_new (char *db_name,
 				      guint32 creator_id)
 {
-	GnomePilotConduitStandardAbs *retval;
+	GtkObject *retval;
 
 	retval =  gtk_object_new (gnome_pilot_conduit_standard_abs_get_type (),
 				  "GnomePilotConduitStandard::db_name", db_name,
-				  "GnomePilotConduitStandard::creator_id", creator_id);
+				  "GnomePilotConduitStandard::creator_id", creator_id,
+				  NULL);
 
-	return GTK_OBJECT (retval);
+	return retval;
 }
 
 /* We have a bunch of marshalling functions that we need to declare.
@@ -399,7 +410,8 @@ gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *
 {
 	GnomePilotConduitStandardAbs *conduit = NULL;
 	LocalRecord *local = NULL;
-	int retval = 0;
+	int result = 0;
+	int err = 0;
 	recordid_t assigned_id;
 
 	g_return_val_if_fail (conduit_standard != NULL, -1);
@@ -407,53 +419,63 @@ gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *
 
 	conduit = GNOME_PILOT_CONDUIT_STANDARD_ABS (conduit_standard);
 
-	/* Open db and prepare conduit */
-	standard_abs_open_db(conduit, dbinfo);
-	if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo) != 0) {
-		g_warning(_("Conduits initialization failed, aborting operation"));
-		return -1;
-	}
-	
-	/* Set the counters for the progress bar, note, total_records is set
-	   in standard_abs_open_db */
-	if (conduit->num_local_records == -1) {
-		conduit->num_local_records = conduit->total_records;
-	}
-	conduit->total_progress += conduit->num_updated_local_records;
-
-	/* nuke all records */
-	if (dlp_DeleteRecord (dbinfo->pilot_socket, dbinfo->db_handle, 1, 0) < 0) {
-		g_warning(_("Unable to delete all records in pilot database, aborting operation."));
-		return -1;
-	}
-
-	while (gnome_pilot_conduit_standard_abs_iterate (conduit, &local) && local) {
-		if (local->archived) {
-			retval = gnome_pilot_conduit_standard_abs_archive_local (conduit, local);
-		} else if (local->attr != GnomePilotRecordDeleted) {
-			PilotRecord *remote;
-			gnome_pilot_conduit_standard_abs_transmit (conduit, local, &remote);
-			if (remote==NULL) {
-			  g_warning(_("Conduit did not return a record"));
-			  break;
-			}
-			gnome_pilot_conduit_standard_abs_set_status (conduit, local, GnomePilotRecordNothing);
-			retval = dlp_WriteRecord(dbinfo->pilot_socket,
-						 dbinfo->db_handle,
-						 remote->secret?dlpRecAttrSecret:0,
-						 remote->ID,
-						 remote->category,
-						 remote->record,
-						 remote->length,
-						 &assigned_id);
-			if (retval>0) gnome_pilot_conduit_standard_abs_set_pilot_id(conduit,local,assigned_id);
-		        gnome_pilot_conduit_standard_abs_free_transmit (conduit, local, &remote);
+	do {
+		
+		/* Open db and prepare conduit */	
+		err = standard_abs_open_db(conduit, dbinfo);
+		
+		if (err < 0) {
+			Result(-1);
 		}
-	}
+		
+		if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo) != 0) {
+			g_warning(_("Conduits initialization failed, aborting operation"));
+			Result(-2);
+		}
+		
+		/* Set the counters for the progress bar, note, total_records is set
+		   in standard_abs_open_db */
+		if (conduit->num_local_records == -1) {
+			conduit->num_local_records = conduit->total_records;
+		}
+		conduit->total_progress += conduit->num_updated_local_records;
+		
+		/* nuke all records */
+		if (dlp_DeleteRecord (dbinfo->pilot_socket, dbinfo->db_handle, 1, 0) < 0) {
+			g_warning(_("Unable to delete all records in pilot database, aborting operation."));
+			Result(-4);
+		}
+		
+		while (gnome_pilot_conduit_standard_abs_iterate (conduit, &local) && local) {
+			if (local->archived) {
+				err = gnome_pilot_conduit_standard_abs_archive_local (conduit, local);
+			} else if (local->attr != GnomePilotRecordDeleted) {
+				PilotRecord *remote;
+				gnome_pilot_conduit_standard_abs_transmit (conduit, local, &remote);
+				if (remote==NULL) {
+					g_warning(_("Conduit did not return a record"));
+					break;
+				}
+				gnome_pilot_conduit_standard_abs_set_status (conduit, local, 
+									     GnomePilotRecordNothing);
+				err = dlp_WriteRecord(dbinfo->pilot_socket,
+							 dbinfo->db_handle,
+							 remote->secret?dlpRecAttrSecret:0,
+							 remote->ID,
+							 remote->category,
+							 remote->record,
+							 remote->length,
+							 &assigned_id);
+				if (err > 0) {
+					gnome_pilot_conduit_standard_abs_set_pilot_id(conduit,local,assigned_id);
+				}
+				gnome_pilot_conduit_standard_abs_free_transmit (conduit, local, &remote);
+			}
+		}
+		
+	} while (0);
 
-	standard_abs_close_db_and_purge_local(conduit,dbinfo);
-
-	return 0;
+	Finish;
 }
 
 static gint
@@ -463,6 +485,8 @@ gnome_pilot_conduit_standard_real_copy_from_pilot   (GnomePilotConduitStandard *
 	GnomePilotConduitStandardAbs *conduit = NULL;
 	unsigned char buffer[0xffff];
 	int index = 0;
+	int result = 0;
+	int err;
 	PilotRecord remote;
 
 	g_return_val_if_fail (conduit_standard != NULL, -1);
@@ -472,49 +496,53 @@ gnome_pilot_conduit_standard_real_copy_from_pilot   (GnomePilotConduitStandard *
 
 	remote.record = buffer;
 
-	/* Open db and prepare conduit */
-	standard_abs_open_db(conduit,dbinfo);
-	if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
-		g_warning(_("Conduits initialization failed, aborting operation"));
-		return -1;
-	}
-
-	/* nuke all records */
-	if (gnome_pilot_conduit_standard_abs_delete_all (conduit) < 0) {
-		g_warning(_("Unable to delete all records in local database, aborting operation."));
-		return -1;
-	}
-
-	/* copy all records */
-	while (dlp_ReadRecordByIndex (dbinfo->pilot_socket,
-				      dbinfo->db_handle,
-				      index,
-				      remote.record,
-				      &remote.ID,
-				      &remote.length,
-				      &remote.attr,
-				      &remote.category) >= 0) {
-		standard_abs_compute_attr_field(&remote);
-		if (remote.archived) {
-			remote.attr = GnomePilotRecordNothing;
-			remote.archived = 0;
-			gnome_pilot_conduit_standard_abs_archive_remote (conduit, 
-									 NULL, 
-									 &remote);
-		} else if (remote.attr != GnomePilotRecordDeleted) {
-			remote.attr = GnomePilotRecordNothing;
-			remote.archived = 0;
-			gnome_pilot_conduit_standard_abs_store_remote (conduit, &remote);
+	do {
+		/* Open db and prepare conduit */
+		err = standard_abs_open_db(conduit,dbinfo);
+		if (err < 0) {
+			Result(-1);
 		}
-		index++;
-		gnome_pilot_conduit_send_progress(GNOME_PILOT_CONDUIT(conduit),
-						  conduit->total_records,
-						  ++conduit->progress);
-	}
 
-	standard_abs_close_db_and_purge_local(conduit,dbinfo);
+		if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
+			g_warning(_("Conduits initialization failed, aborting operation"));
+			Result(-2);
+		}
+		
+		/* nuke all records */
+		if (gnome_pilot_conduit_standard_abs_delete_all (conduit) < 0) {
+			g_warning(_("Unable to delete all records in local database, aborting operation."));
+			Result(-3);
+		}
+		
+		/* copy all records */
+		while (dlp_ReadRecordByIndex (dbinfo->pilot_socket,
+					      dbinfo->db_handle,
+					      index,
+					      remote.record,
+					      &remote.ID,
+					      &remote.length,
+					      &remote.attr,
+					      &remote.category) >= 0) {
+			standard_abs_compute_attr_field(&remote);
+			if (remote.archived) {
+				remote.attr = GnomePilotRecordNothing;
+				remote.archived = 0;
+				gnome_pilot_conduit_standard_abs_archive_remote (conduit, 
+										 NULL, 
+										 &remote);
+			} else if (remote.attr != GnomePilotRecordDeleted) {
+				remote.attr = GnomePilotRecordNothing;
+				remote.archived = 0;
+				gnome_pilot_conduit_standard_abs_store_remote (conduit, &remote);
+			}
+			index++;
+			gnome_pilot_conduit_send_progress(GNOME_PILOT_CONDUIT(conduit),
+							  conduit->total_records,
+							  ++conduit->progress);
+		}		
+	} while (0);
 
-	return 0;
+	Finish;
 }
 
 static gint
@@ -522,23 +550,28 @@ gnome_pilot_conduit_standard_real_merge_to_pilot    (GnomePilotConduitStandard *
 						     GnomePilotDBInfo  *dbinfo)
 {
 	GnomePilotConduitStandardAbs *conduit = NULL;
+	int result = 0;
+	int err = 0;
 
 	g_return_val_if_fail (conduit_standard != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD_ABS (conduit_standard), -1);	
 
 	conduit = GNOME_PILOT_CONDUIT_STANDARD_ABS (conduit_standard);
 
-	/* Open db and prepare conduit */
-	standard_abs_open_db(conduit, dbinfo);
-	if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
-		g_warning(_("Conduits initialization failed, aborting operation"));
-		return -1;
-	}
-	standard_abs_merge_to_remote(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncToRemote );
-
-	standard_abs_close_db_and_purge_local(conduit,dbinfo);
+	do {
+		/* Open db and prepare conduit */
+		err = standard_abs_open_db(conduit, dbinfo);
+		if (err < 0) {
+			Result(-1);
+		}
+		if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
+			g_warning(_("Conduits initialization failed, aborting operation"));
+			Result(-2);
+		}
+		standard_abs_merge_to_remote(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncToRemote );
+	} while (0);
 	
-	return 0;
+	Finish;
 }
 
 static gint
@@ -546,30 +579,36 @@ gnome_pilot_conduit_standard_real_merge_from_pilot  (GnomePilotConduitStandard *
 						     GnomePilotDBInfo  *dbinfo)
 {
 	GnomePilotConduitStandardAbs *conduit = NULL;
+	int result = 0;
+	int err = 0;
 
 	g_return_val_if_fail (conduit_standard != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD_ABS (conduit_standard), -1);
 
 	conduit = GNOME_PILOT_CONDUIT_STANDARD_ABS (conduit_standard);
 
-	/* Open db and prepare conduit */
-	standard_abs_open_db(conduit, dbinfo);
-	if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
-		g_warning(_("Conduits initialization failed, aborting operation"));
-		return -1;
-	}
-	standard_abs_merge_to_local(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncToLocal );
+	do {
+		/* Open db and prepare conduit */
+		err = standard_abs_open_db(conduit, dbinfo);
+		if (err < 0) {
+			Result(-1);
+		}
+		if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
+			g_warning(_("Conduits initialization failed, aborting operation"));
+			Result(-2);
+		}
+		standard_abs_merge_to_local(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncToLocal );
+	} while (0);		
 
-	standard_abs_close_db_and_purge_local(conduit,dbinfo);
-
-	return 0;
+	Finish;
 }
 
 static gint
 gnome_pilot_conduit_standard_real_synchronize       (GnomePilotConduitStandard *conduit_standard,
 						     GnomePilotDBInfo  *dbinfo)
 {
-	int retval = 0;
+	int result = 0;
+	int err = 0;
 	GnomePilotConduitStandardAbs *conduit = NULL;
 
 	g_return_val_if_fail (conduit_standard != NULL, -1);
@@ -577,55 +616,58 @@ gnome_pilot_conduit_standard_real_synchronize       (GnomePilotConduitStandard *
 
 	conduit = GNOME_PILOT_CONDUIT_STANDARD_ABS(conduit_standard);
 
-	/* Open db and prepare conduit */
-	standard_abs_open_db(conduit, dbinfo);
-	if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
-		g_warning(_("Conduits initialization failed, aborting operation"));
-		return -1;
-	}
-
-	/* Set the counters for the progress bar, note, total_records is set
-	   in standard_abs_open_db */
-	conduit->total_progress = 0;
-	if (conduit->num_local_records == -1) {
-		conduit->num_local_records = conduit->total_records;
-	}
-	conduit->total_progress += conduit->num_updated_local_records;
-	if (conduit->num_updated_local_records == -1) {
-		conduit->num_updated_local_records = conduit->total_records;
-	}
-	conduit->total_progress += conduit->num_updated_local_records;
-	if (conduit->num_new_local_records == -1) {
-		conduit->num_new_local_records = conduit->total_records;
-	}
-	conduit->total_progress += conduit->num_deleted_local_records;
-	if (conduit->num_deleted_local_records == -1) {
-		conduit->num_deleted_local_records = conduit->total_records;
-	}
-	conduit->total_progress += conduit->num_deleted_local_records;
-
-	/* Decide on fast sync or slow sync */
-	/* I think the following if is bad juju */
-	if (1 || PI_DBINFO (dbinfo)->backupDate < PI_DBINFO (dbinfo)->modifyDate) {
-		if (conduit_standard->slow==FALSE && gpilot_sync_pc_match (dbinfo)==TRUE) {
-			retval = FastSync(dbinfo->pilot_socket, dbinfo->db_handle, conduit);
-		} else {
-			conduit->total_progress += conduit->total_records;
-			retval = SlowSync(dbinfo->pilot_socket, dbinfo->db_handle, conduit);
+	do {
+		/* Open db and prepare conduit */
+		err = standard_abs_open_db(conduit, dbinfo);
+		if (err < 0) {
+			Result(-1);
 		}
-	}
+		if (gnome_pilot_conduit_standard_abs_pre_sync(conduit, dbinfo)!=0) {
+			g_warning(_("Conduits initialization failed, aborting operation"));
+			Result(-2);
+		}
+		
+		/* Set the counters for the progress bar, note, total_records is set
+		   in standard_abs_open_db */
+		conduit->total_progress = 0;
+		if (conduit->num_local_records == -1) {
+			conduit->num_local_records = conduit->total_records;
+		}
+		conduit->total_progress += conduit->num_updated_local_records;
+		if (conduit->num_updated_local_records == -1) {
+			conduit->num_updated_local_records = conduit->total_records;
+		}
+		conduit->total_progress += conduit->num_updated_local_records;
+		if (conduit->num_new_local_records == -1) {
+			conduit->num_new_local_records = conduit->total_records;
+		}
+		conduit->total_progress += conduit->num_deleted_local_records;
+		if (conduit->num_deleted_local_records == -1) {
+			conduit->num_deleted_local_records = conduit->total_records;
+		}
+		conduit->total_progress += conduit->num_deleted_local_records;
 
-	/* now disable the slow=true setting, so it doens't pass on to next time */
-	if (conduit_standard->slow==TRUE)
-		conduit_standard->slow = FALSE;
+		/* Decide on fast sync or slow sync */
+		/* I think the following if is bad juju */
+		if (1 || PI_DBINFO (dbinfo)->backupDate < PI_DBINFO (dbinfo)->modifyDate) {
+			if (conduit_standard->slow==FALSE && gpilot_sync_pc_match (dbinfo)==TRUE) {
+				result = FastSync(dbinfo->pilot_socket, dbinfo->db_handle, conduit);
+			} else {
+				conduit->total_progress += conduit->total_records;
+				result = SlowSync(dbinfo->pilot_socket, dbinfo->db_handle, conduit);
+			}
+		}
+		
+		/* now disable the slow=true setting, so it doens't pass on to next time */
+		if (conduit_standard->slow==TRUE)
+			conduit_standard->slow = FALSE;
+		
+		standard_abs_merge_to_remote(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncBothWays );
+		standard_abs_check_locally_deleted_records(conduit, dbinfo->pilot_socket, 
+							   dbinfo->db_handle, SyncBothWays);
+	} while (0);
 
-	standard_abs_merge_to_remote(conduit, dbinfo->pilot_socket, dbinfo->db_handle, SyncBothWays );
-	standard_abs_check_locally_deleted_records(conduit, dbinfo->pilot_socket, 
-						   dbinfo->db_handle, SyncBothWays);
-
-	standard_abs_close_db_and_purge_local(conduit,dbinfo);
-
-	return retval;
+	Finish;
 }
 
 gint
@@ -1588,11 +1630,12 @@ gpilot_sync_pc_match(GnomePilotDBInfo *dbinfo)
 	return FALSE;
 }
 
-void
+gint
 standard_abs_open_db(GnomePilotConduitStandardAbs *conduit,
 		     GnomePilotDBInfo *dbinfo)
 {
 	gchar *name;
+	int err;
   
 	g_assert(conduit!=NULL);
 	g_assert(dbinfo!=NULL);
@@ -1600,29 +1643,37 @@ standard_abs_open_db(GnomePilotConduitStandardAbs *conduit,
 	name = g_strdup(gnome_pilot_conduit_standard_get_db_name (GNOME_PILOT_CONDUIT_STANDARD(conduit)));
 	if (conduit->db_open_mode!=0) {
 		g_message("gpilotd: open_db: opening with %d\n",conduit->db_open_mode);
-		dlp_OpenDB (dbinfo->pilot_socket, 0, conduit->db_open_mode, 
-			    name, 
-			    &(dbinfo->db_handle));
+		err = dlp_OpenDB (dbinfo->pilot_socket, 0, conduit->db_open_mode, 
+				  name, 
+				  &(dbinfo->db_handle));
 	} else {
-		dlp_OpenDB (dbinfo->pilot_socket, 0, dlpOpenReadWrite,
-			    name,
-			    &(dbinfo->db_handle));
+		err = dlp_OpenDB (dbinfo->pilot_socket, 0, dlpOpenReadWrite,
+				  name,
+				  &(dbinfo->db_handle));
 	}
 
-	dlp_ReadOpenDBInfo(dbinfo->pilot_socket,dbinfo->db_handle,&conduit->total_records);
+	if (err < 0) {
+		g_message ("gpilotd: open_db, error %s", dlp_strerror (err));
+	} else {		
+		dlp_ReadOpenDBInfo(dbinfo->pilot_socket,dbinfo->db_handle,&conduit->total_records);
+	}
 	g_free(name);
+
+	return err;
 }
 
 void
 standard_abs_close_db_and_purge_local(GnomePilotConduitStandardAbs *conduit,
-				      GnomePilotDBInfo *dbinfo)
+				      GnomePilotDBInfo *dbinfo,
+				      gboolean clean)
 {
 	g_assert(conduit!=NULL);
 	g_assert(dbinfo!=NULL);
 
-	dlp_CleanUpDatabase (dbinfo->pilot_socket, dbinfo->db_handle);
-	gnome_pilot_conduit_standard_abs_purge (conduit);
-
-	dlp_ResetSyncFlags (dbinfo->pilot_socket, dbinfo->db_handle);
+	if (clean) {
+		dlp_CleanUpDatabase (dbinfo->pilot_socket, dbinfo->db_handle);
+		gnome_pilot_conduit_standard_abs_purge (conduit);		
+		dlp_ResetSyncFlags (dbinfo->pilot_socket, dbinfo->db_handle);
+	}
 	dlp_CloseDB (dbinfo->pilot_socket, dbinfo->db_handle);
 }
